/****************************************************************************
 *                                                                          *
 *  COPYRIGHT (c) 1994 BY                                                   *
 *  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.                  *
 *  ALL RIGHTS RESERVED.                                                    *
 *                                                                          *
 *  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   *
 *  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   *
 *  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY  OTHER   *
 *  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   *
 *  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   *
 *  TRANSFERRED.                                                            *
 *                                                                          *
 *  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   *
 *  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   *
 *  CORPORATION.                                                            *
 *                                                                          *
 *  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   *
 *  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
 *                                                                          *
 ****************************************************************************
 *              Copyright 1994 Digital Equipment Corporation
 *                         All Rights Reserved
 *
 * Permission to use, copy, and modify this software and its documentation is
 * hereby granted only under the following terms and conditions.  Both the
 * above copyright notice and this permission notice must appear in all copies
 * of the software, derivative works or modified versions, and any portions
 * thereof, and both notices must appear in supporting documentation.
 *
 * Users of this software agree to the terms and conditions set forth herein,
 * and hereby grant back to Digital a non-exclusive, unrestricted, royalty-
 * free right and license under any changes, enhancements or extensions 
 * made to the core functions of the software, including but not limited to 
 * those affording compatibility with other hardware or software 
 * environments, but excluding applications which incorporate this software.
 * Users further agree to use their best efforts to return to Digital any
 * such changes, enhancements or extensions that they make and inform Digital
 * of noteworthy uses of this software.  Correspondence should be provided
 * to Digital at:
 *
 *                       Director of Licensing
 *                       Western Research Laboratory
 *                       Digital Equipment Corporation
 *                       250 University Ave
 *                       Palo Alto, California  94301
 *
 * This software may be distributed (but not offered for sale or transferred
 * for compensation) to third parties, provided such third parties agree to
 * abide by the terms and conditions of this notice.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND DIGITAL EQUIPMENT CORP. DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL DIGITAL EQUIPMENT
 * CORPORATION BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */

/*
**  map.h	- Interface file for heap address tracking module.
**
**	This module keeps track of address ranges in a program's heap.
**	The calling program must notify this module whenever a memory
**	segment is allocated, free'd, or sbrk'd.  The caller may then
**	inquire about the state of specific heap addresses.
**
**	This module does no error checking for the integrity of the heap
**	allocations.  That is the responsibility of the caller.
*/

#ifndef MAP_H
#define MAP_H

typedef caddr_t mapaddr_t;	/* address in program's heap */

/*
 * State of memory segment.
 */
typedef enum {
    MAPST_ALLOCATED,	/* segment allocated */
    MAPST_FREED,	/* segment was free'd (either with free or brk/sbrk) */
    MAPST_SBRKD,	/* segment allocated with brk/sbrk */
    MAPST_UNALLOCATED	/* segment never allocated */
} mapstate_t;

/*
 * Additional information about allocated, free'd, or sbrk'd segment.
 */
typedef struct {
    unsigned long 	size;	/* size in bytes */
    caddr_t           	pc;	/* PC where segment was allocated, free'd */
    unsigned           	file;	/* file where segment was allocated, free'd */
    unsigned           	line;	/* line where segment was allocated, free'd */
    unsigned long	seq;	/* sequence number */
} mapinfo_t;


/*
 * Interface routines.
 */
void          	map_alloc(
			mapaddr_t,
			unsigned long size,
			caddr_t pc,
			unsigned file,
			unsigned line,
			unsigned long seq);
void          	map_free(
			mapaddr_t,
			unsigned long size,
			caddr_t pc,
			unsigned file,
			unsigned line,
			unsigned long seq);
void          	map_sbrk(
			mapaddr_t,
			long size,
			caddr_t pc,
			unsigned file,
			unsigned line,
			unsigned long seq);
mapstate_t 	map_find(
			mapaddr_t,
			const mapinfo_t **);
mapstate_t 	map_find_containing(
			mapaddr_t,
			const mapinfo_t **);
mapstate_t 	map_find_previous(
			mapaddr_t addr,
			mapaddr_t * pstart,
			const mapinfo_t ** ppbuf);
mapstate_t 	map_find_following(
			mapaddr_t addr,
			mapaddr_t * pstart,
			const mapinfo_t ** ppbuf);
unsigned long 	map_total(void);

#endif /* MAP_H */
