/****************************************************************************
 *                                                                          *
 *  COPYRIGHT (c) 1994 BY                                                   *
 *  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.                  *
 *  ALL RIGHTS RESERVED.                                                    *
 *                                                                          *
 *  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   *
 *  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   *
 *  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY  OTHER   *
 *  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   *
 *  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   *
 *  TRANSFERRED.                                                            *
 *                                                                          *
 *  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   *
 *  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   *
 *  CORPORATION.                                                            *
 *                                                                          *
 *  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   *
 *  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
 *                                                                          *
 ****************************************************************************
 *              Copyright 1994 Digital Equipment Corporation
 *                         All Rights Reserved
 *
 * Permission to use, copy, and modify this software and its documentation is
 * hereby granted only under the following terms and conditions.  Both the
 * above copyright notice and this permission notice must appear in all copies
 * of the software, derivative works or modified versions, and any portions
 * thereof, and both notices must appear in supporting documentation.
 *
 * Users of this software agree to the terms and conditions set forth herein,
 * and hereby grant back to Digital a non-exclusive, unrestricted, royalty-
 * free right and license under any changes, enhancements or extensions 
 * made to the core functions of the software, including but not limited to 
 * those affording compatibility with other hardware or software 
 * environments, but excluding applications which incorporate this software.
 * Users further agree to use their best efforts to return to Digital any
 * such changes, enhancements or extensions that they make and inform Digital
 * of noteworthy uses of this software.  Correspondence should be provided
 * to Digital at:
 *
 *                       Director of Licensing
 *                       Western Research Laboratory
 *                       Digital Equipment Corporation
 *                       250 University Ave
 *                       Palo Alto, California  94301
 *
 * This software may be distributed (but not offered for sale or transferred
 * for compensation) to third parties, provided such third parties agree to
 * abide by the terms and conditions of this notice.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND DIGITAL EQUIPMENT CORP. DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL DIGITAL EQUIPMENT
 * CORPORATION BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */

#ifdef STATS

/*
**  stat.c	- Definition file for statistics gathering routines.
**
**	This module reports statistics about the binary tree.  STATS
**	must be defined to enable this feature.
*/

#include <stdio.h>
#include "interface.h"


unsigned long 	_stat_num_nodes;
unsigned long 	_stat_num_insert;
unsigned long 	_stat_num_remove;
unsigned long 	_stat_total_depth_1;
unsigned long 	_stat_total_depth_2;
unsigned long 	_stat_total_depth_3;
double   	_stat_total_num_nodes;
unsigned long 	_stat_num_searches;


/*
**  Report the statistics gathered thus far.
*/
void stat_report(void)
{
    fprintf(HeapFile, "Statistics:\n");
    fprintf(HeapFile, "\tTotal searches:           %ld\n",
	_stat_num_searches);
    fprintf(HeapFile, "\tTotal insertions:         %ld\n",
	_stat_num_insert);
    fprintf(HeapFile, "\tTotal removals:           %ld\n",
	_stat_num_remove);
    fprintf(HeapFile, "\tTotal child 1 recursions: %ld\n",
	_stat_total_depth_1);
    fprintf(HeapFile, "\tTotal child 2 recursions: %ld\n",	
	_stat_total_depth_2);
    fprintf(HeapFile, "\tTotal child 3 recursions: %ld\n",	
	_stat_total_depth_3);
    fprintf(HeapFile, "\n");

    if (_stat_num_searches) {
	fprintf(HeapFile, "\tAvg. number nodes:              %g\n",
	    _stat_total_num_nodes/(double)_stat_num_searches);
	fprintf(HeapFile, "\tAvg. number child 1 recursions: %g\n",
	    (double)_stat_total_depth_1/(double)_stat_num_searches);
	fprintf(HeapFile, "\tAvg. number child 2 recursions: %g\n",
	    (double)_stat_total_depth_2/(double)_stat_num_searches);
	fprintf(HeapFile, "\tAvg. number child 3 recursions: %g\n",
	    (double)_stat_total_depth_3/(double)_stat_num_searches);
    }
}


#endif /* STATS */
