/****************************************************************************
 *                                                                          *
 *  COPYRIGHT (c) 1994 BY                                                   *
 *  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.                  *
 *  ALL RIGHTS RESERVED.                                                    *
 *                                                                          *
 *  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   *
 *  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   *
 *  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY  OTHER   *
 *  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   *
 *  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   *
 *  TRANSFERRED.                                                            *
 *                                                                          *
 *  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   *
 *  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   *
 *  CORPORATION.                                                            *
 *                                                                          *
 *  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   *
 *  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
 *                                                                          *
 ****************************************************************************
 *              Copyright 1994 Digital Equipment Corporation
 *                         All Rights Reserved
 *
 * Permission to use, copy, and modify this software and its documentation is
 * hereby granted only under the following terms and conditions.  Both the
 * above copyright notice and this permission notice must appear in all copies
 * of the software, derivative works or modified versions, and any portions
 * thereof, and both notices must appear in supporting documentation.
 *
 * Users of this software agree to the terms and conditions set forth herein,
 * and hereby grant back to Digital a non-exclusive, unrestricted, royalty-
 * free right and license under any changes, enhancements or extensions 
 * made to the core functions of the software, including but not limited to 
 * those affording compatibility with other hardware or software 
 * environments, but excluding applications which incorporate this software.
 * Users further agree to use their best efforts to return to Digital any
 * such changes, enhancements or extensions that they make and inform Digital
 * of noteworthy uses of this software.  Correspondence should be provided
 * to Digital at:
 *
 *                       Director of Licensing
 *                       Western Research Laboratory
 *                       Digital Equipment Corporation
 *                       250 University Ave
 *                       Palo Alto, California  94301
 *
 * This software may be distributed (but not offered for sale or transferred
 * for compensation) to third parties, provided such third parties agree to
 * abide by the terms and conditions of this notice.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND DIGITAL EQUIPMENT CORP. DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL DIGITAL EQUIPMENT
 * CORPORATION BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "platform.h"

#define MAXFD 256
#define MAXSIZE 16384

struct {
  long count;
  long alignedAddress[8];
  long time;
  long bytes;
} writeSizes[MAXSIZE],readSizes[MAXSIZE],writeOther,readOther;

struct fileType {
  long opens;
  long reads;
  long writes;
  long seeks;
  long bytes;
} files[MAXFD];

int lastSize = 0;

unsigned int ccLast = 0;

void AfterOpen(long fd)
{
  if (fd >= 0 && fd < MAXFD)
    {
      files[fd].opens++;
    }
}

void BeforeSeek(long fd)
{
  if (fd >= 0 && fd < MAXFD)
    {
      files[fd].seeks++;
    }
}

void BeforeWrite(long fd, long address, long size, long cc)
{
  if (size < MAXSIZE)
    {
      writeSizes[size].count++;
      writeSizes[size].alignedAddress[address & 0x7]++;
      writeSizes[size].bytes += size;
    }
  else
    {
      writeOther.count++;
      writeOther.alignedAddress[address & 0x7]++;
      writeOther.bytes += size;
    }
  if (fd >= 0 && fd < MAXFD)
    {
      files[fd].bytes += size;
      files[fd].writes++;
    }
  ccLast = cc;
  lastSize = size;
}

void AfterWrite(long cc)
{
  unsigned int ccInt;

  ccInt = cc;
  if (lastSize < MAXSIZE)
    {
      writeSizes[lastSize].time += ccInt - ccLast;
    }
  else
    {
      writeOther.time += ccInt - ccLast;
    }
  ccLast = ccInt;
}

void BeforeRead(long fd, long address, long size, long cc)
{
  if (size < MAXSIZE)
    {
      readSizes[size].count++;
      readSizes[size].alignedAddress[address & 0x7]++;
      readSizes[size].bytes += size;
    }
  else
    {
      readOther.count++;
      readOther.alignedAddress[address & 0x7]++;
      readOther.bytes += size;
    }
  if (fd >= 0 && fd < MAXFD)
    {
      files[fd].bytes += size;
      files[fd].reads++;
    }
  ccLast = cc;
  lastSize = size;
}

void AfterRead(long cc)
{
  int ccInt;

  ccInt = cc;
  if (lastSize < MAXSIZE)
    {
      readSizes[lastSize].time += ccInt - ccLast;
    }
  else
    {
      readOther.time += ccInt - ccLast;
    }
  ccLast = ccInt;
}

FILE *OpenUnique(char *fileName, char *type)
{
  FILE *file;
  char Name[200];

  if (getenv("ATOMUNIQUE") != NULL)
    sprintf(Name,"%s.%d",fileName,getpid());
  else
    strcpy(Name,fileName);

  file = fopen(Name,type);
  if (file == NULL)
    {
      fprintf(stderr,"Atom: can't open %s for %s\n",Name, type);
      exit(1);
    }
  return(file);
}

FILE *file;

void GetPlatform(PlatformType *platform)
{
  FILE *filePlatform;
  char model[100],buffer[100];
  int i;
  char *flag;

  filePlatform = popen("/usr/sbin/sizer -c","r");
  assert(filePlatform != NULL);
  flag = fgets(buffer,100,filePlatform);
  sscanf(buffer,"cpu \"%[^\"]s",model);

  for (i = 0; strcmp(model,platforms[i].model)!=0 &&
              platforms[i].model[0] != '\0'; i++);
  if (platforms[i].model[0] == '\0')
    {
      fprintf(file,"Model %s not found\n",model);
      i = 0;
    }
  fprintf(file,"Using model %s platform parameters\n\n",platforms[i].model);
  *platform = platforms[i];
  fclose(filePlatform);
}


PlatformType platform;

void OpenFile()
{
  int i;
  
  file = OpenUnique("io.out","w");
  GetPlatform(&platform);
}

void PrintResults()
{
  int i;
  
  fprintf(file,
  "-----------------------------------------------------------------------\n");
  fprintf(file,"Input/Output Summary\n");
  fprintf(file,
  "-----------------------------------------------------------------------\n");
  fprintf(file,"\n%10s %10s %10s %10s %10s %13s\n", 
	  "File Desc","Opens","Reads","Writes","Seeks","Bytes");

  for (i = 0; i < MAXFD; i++)
    {
      if (files[i].opens || files[i].bytes)
	{
	  fprintf(file,"%10ld %10ld %10ld %10ld %10ld %13ld\n", 
		  i, files[i].opens, files[i].reads, files[i].writes,
		  files[i].seeks, files[i].bytes);
	}
    }

  fprintf(file,
  "-----------------------------------------------------------------------\n");
  fprintf(file,"Read Summary\n");
  fprintf(file,
  "-----------------------------------------------------------------------\n");
  fprintf(file,"\n%13s %13s %13s %13s %13s\n", "Size","Count","Bytes",
	  "Time(ms)", "Rate");
  for (i = 0; i < MAXSIZE; i++)
    {
      if (readSizes[i].count != 0)
	{
	  fprintf(file,"%13ld %13ld %13ld %13.1f %13.1f\n",
		  i, readSizes[i].count,
		  readSizes[i].bytes, 
		  (platform.cycleTime * readSizes[i].time) / 1000000.0,
		  readSizes[i].bytes / 
		  (platform.cycleTime * readSizes[i].time) / 1000000000.0);
	}
    }
  if (readOther.count != 0)
    {
      fprintf(file,"%13s %13ld %13ld %13.1f %13.1f\n",
	      "Other", readOther.count,
	      readOther.bytes, 
	      (platform.cycleTime * readOther.time) / 1000000.0,
	      readOther.bytes / 
	      ((platform.cycleTime * readOther.time) / 1000000000.0));
    }

  fprintf(file,
  "-----------------------------------------------------------------------\n");
  fprintf(file,"Read Address Alignment\n");
  fprintf(file,
  "-----------------------------------------------------------------------\n");
  fprintf(file,"\n%13s %13s %13s %13s %13s\n",
	  "Size","Quad", "Long", "Word", "Byte");
  for (i = 0; i < MAXSIZE; i++)
    {
      if (readSizes[i].count != 0)
	{
	  fprintf(file,"%13ld %13ld %13ld %13ld %13ld\n",i,
		  readSizes[i].alignedAddress[0],
		  readSizes[i].alignedAddress[4],
		  readSizes[i].alignedAddress[2] +
		          readSizes[i].alignedAddress[6],
		  readSizes[i].alignedAddress[1] +
		          readSizes[i].alignedAddress[3] +
		          readSizes[i].alignedAddress[5] +
		          readSizes[i].alignedAddress[7]);
	}
    }
  if (readOther.count != 0)
    {
      fprintf(file,"%13s %13ld %13ld %13ld %13ld\n","Other",
	      readOther.alignedAddress[0],
	      readOther.alignedAddress[4],
	      readOther.alignedAddress[2] +
		          readOther.alignedAddress[6],
	      readOther.alignedAddress[1] +
		          readOther.alignedAddress[3] +
		          readOther.alignedAddress[5] +
		          readOther.alignedAddress[7]);
    }

  fprintf(file,
  "-----------------------------------------------------------------------\n");
  fprintf(file,"Write Summary\n");
  fprintf(file,
  "-----------------------------------------------------------------------\n");
  fprintf(file,"\n%13s %13s %13s %13s %13s\n", "Size","Count","Bytes",
	  "Time(ms)", "Rate");
  for (i = 0; i < MAXSIZE; i++)
    {
      if (writeSizes[i].count != 0)
	{
	  fprintf(file,"%13ld %13ld %13ld %13.1f %13.1f\n",
		  i, writeSizes[i].count,
		  writeSizes[i].bytes, 
		  (platform.cycleTime * writeSizes[i].time) / 1000000.0,
		  writeSizes[i].bytes / 
		  (platform.cycleTime * writeSizes[i].time) / 1000000000.0);
	}
    }
  if (writeOther.count != 0)
    {
      fprintf(file,"%13s %13ld %13ld %13.1lf %13.1lf\n","Other",
	      writeOther.count,
	      writeOther.bytes, 
	      (platform.cycleTime * writeOther.time) / 1000000.0,
	      writeOther.bytes / 
	      (platform.cycleTime * writeOther.time) / 1000000000.0);
    }

  fprintf(file,
  "-----------------------------------------------------------------------\n");
  fprintf(file,"Write Address Alignment\n");
  fprintf(file,
  "-----------------------------------------------------------------------\n");
  fprintf(file,"\n%13s %13s %13s %13s %13s\n",
	  "Size","Quad", "Long", "Word", "Byte");
  for (i = 0; i < MAXSIZE; i++)
    {
      if (writeSizes[i].count != 0)
	{
	  fprintf(file,"%13ld %13ld %13ld %13ld %13ld\n",i,
		  writeSizes[i].alignedAddress[0],
		  writeSizes[i].alignedAddress[4],
		  writeSizes[i].alignedAddress[2] +
		          writeSizes[i].alignedAddress[6],
		  writeSizes[i].alignedAddress[1] +
		          writeSizes[i].alignedAddress[3] +
		          writeSizes[i].alignedAddress[5] +
		          writeSizes[i].alignedAddress[7]);
	}
    }

  if (writeOther.count != 0)
    {
      fprintf(file,"%13s %13ld %13ld %13ld %13ld\n","Other",
	      writeOther.alignedAddress[0],
	      writeOther.alignedAddress[4],
	      writeOther.alignedAddress[2] +
		          writeOther.alignedAddress[6],
	      writeOther.alignedAddress[1] +
		          writeOther.alignedAddress[3] +
		          writeOther.alignedAddress[5] +
		          writeOther.alignedAddress[7]);
    }
  fclose(file);
}











