/****************************************************************************
 *                                                                          *
 *  COPYRIGHT (c) 1994 BY                                                   *
 *  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.                  *
 *  ALL RIGHTS RESERVED.                                                    *
 *                                                                          *
 *  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   *
 *  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   *
 *  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY  OTHER   *
 *  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   *
 *  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   *
 *  TRANSFERRED.                                                            *
 *                                                                          *
 *  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   *
 *  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   *
 *  CORPORATION.                                                            *
 *                                                                          *
 *  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   *
 *  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
 *                                                                          *
 ****************************************************************************
 *              Copyright 1994 Digital Equipment Corporation
 *                         All Rights Reserved
 *
 * Permission to use, copy, and modify this software and its documentation is
 * hereby granted only under the following terms and conditions.  Both the
 * above copyright notice and this permission notice must appear in all copies
 * of the software, derivative works or modified versions, and any portions
 * thereof, and both notices must appear in supporting documentation.
 *
 * Users of this software agree to the terms and conditions set forth herein,
 * and hereby grant back to Digital a non-exclusive, unrestricted, royalty-
 * free right and license under any changes, enhancements or extensions 
 * made to the core functions of the software, including but not limited to 
 * those affording compatibility with other hardware or software 
 * environments, but excluding applications which incorporate this software.
 * Users further agree to use their best efforts to return to Digital any
 * such changes, enhancements or extensions that they make and inform Digital
 * of noteworthy uses of this software.  Correspondence should be provided
 * to Digital at:
 *
 *                       Director of Licensing
 *                       Western Research Laboratory
 *                       Digital Equipment Corporation
 *                       250 University Ave
 *                       Palo Alto, California  94301
 *
 * This software may be distributed (but not offered for sale or transferred
 * for compensation) to third parties, provided such third parties agree to
 * abide by the terms and conditions of this notice.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND DIGITAL EQUIPMENT CORP. DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL DIGITAL EQUIPMENT
 * CORPORATION BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */
#include <stdio.h>
#include <stdlib.h>
#include "lpipe.h"

BasicBlockData *results;
long instructions_total;
long packedcycles_total;
long cycles_total;

long loop_start_PC;
long instructions_loop;
long packedcycles_loop;
long cycles_loop;

void BlockCount(long number)
{
  results[number].ExecutionCount++;
}

FILE *OpenUnique(char *fileName, char *type)
{
  FILE *file;
  char Name[200];

  if (getenv("ATOMUNIQUE") != NULL)
    sprintf(Name,"%s.%d",fileName,getpid());
  else
    strcpy(Name,fileName);

  file = fopen(Name,type);
  if (file == NULL)
    {
      fprintf(stderr,"Atom: can't open %s for %s\n",Name, type);
      exit(1);
    }
  return(file);
}

FILE *file;
void OpenFile(char *data)
{
  file = OpenUnique("lpipe.dynamic","w");
  results =  (BasicBlockData *) data;
}

void CloseFile()
{
  fclose(file);
}

#define NLOOPS 10

typedef struct
{ 
  long StartPC;
  long EndPC;

  long NInstructions;		/* Instructions in block */
  long NPackedCycles;           /* Cycles in block with no stalls at
				   all */
  long NCycles;			/* Cycles in block without dynamic stalls */
  long NPipeStall;		
} LoopData;

LoopData ldata[NLOOPS];

InitLoop() {
    int i;
    for (i = 0; i < NLOOPS; i++) {
	ldata[i].NInstructions = 0;
	ldata[i].NCycles = 0;
	ldata[i].NPackedCycles = 0;
	ldata[i].NPipeStall = 0;
    }
}

PrintLoop() {
    int i;
    fprintf(file, "\n\n\n");
    fprintf(file,
	    "----------------------------------------------------------------------------------\n");
    fprintf(file, "inner loops or blocks with biggest contributions to total pipeline stalls\n");
    fprintf(file,
	    "----------------------------------------------------------------------------------\n");
    fprintf(file,"%11s %11s   %12s %13s %5s %13s %5s\n",
	    "StartPC", "EndPC",
	    "Instructions", "TotalCycles", "CPI", "PipeStalls",
	    "SPct");
    for (i = 0; i < NLOOPS; i++) {
	if (ldata[i].NCycles == 0) continue;
	fprintf(file,
		"0x%9lx:0x%9lx   %12ld %13ld %5.2f %13ld %5.2f\n",
		ldata[i].StartPC, ldata[i].EndPC,
		ldata[i].NInstructions,
		ldata[i].NCycles,
		ldata[i].NCycles/(float)ldata[i].NInstructions,
		ldata[i].NPipeStall,
		100*ldata[i].NPipeStall/(float)cycles_total
		);
    }
}

EnterLoop(long startPC, long endPC, long instructions, long packedcycles,
	  long cycles) {
    int i;
    long pipestall = cycles - packedcycles;
    if (pipestall <= ldata[NLOOPS-1].NPipeStall) return;
    for (i = NLOOPS-1; i > 0; i--) {
	if (pipestall <= ldata[i-1].NPipeStall) break;
	ldata[i].StartPC = ldata[i-1].StartPC;
	ldata[i].EndPC = ldata[i-1].EndPC;
	ldata[i].NCycles = ldata[i-1].NCycles;
	ldata[i].NPackedCycles = ldata[i-1].NPackedCycles;
	ldata[i].NInstructions = ldata[i-1].NInstructions;
	ldata[i].NPipeStall = ldata[i-1].NPipeStall;
    }
    ldata[i].StartPC = startPC;
    ldata[i].EndPC = endPC;
    ldata[i].NInstructions = instructions;
    ldata[i].NCycles = cycles;
    ldata[i].NPackedCycles = packedcycles;
    ldata[i].NPipeStall = cycles - packedcycles;
}



void PrintProc(long numberFirst, long number, char *name, char *fileName)
{
    long i;
    int loop_on = 0;
    int first = 1;
    for (i = numberFirst; i < number; i++) {
        if (results[i].NInstructions > 0)
	  {
	    if (first) {
	        fprintf(file,
              "----------------------------------------------------------------------------------\n");
	        fprintf(file,"%s : %s\n",name,fileName);
	        fprintf(file,
	    "----------------------------------------------------------------------------------\n");
	        fprintf(file,"  %5s %11s %4s %4s %4s   %4s %12s %13s %13s\n",
		      "Line", "PC", "Inst", "Pack", "Cycl", "CPI",
		      "Count", "TotalCycles", "PipeStalls");
	        first = 0;
	      }
	    fprintf(file,
		 "  %5d 0x%9lx %4d %4d %4d  %5.2f %12ld %13ld %13ld\n",
		 results[i].LineNumber,results[i].PC,results[i].NInstructions, 
		 results[i].NPackedCycles, results[i].NCycles,
		 results[i].NCycles/(float)results[i].NInstructions,
		 results[i].ExecutionCount,
		 results[i].NCycles*results[i].ExecutionCount,
		 ((results[i].NCycles-results[i].NPackedCycles) *
		 results[i].ExecutionCount)
		 );
	  }
	instructions_total += (results[i].NInstructions *
			       results[i].ExecutionCount);
	packedcycles_total += (results[i].NPackedCycles *
			       results[i].ExecutionCount);
	cycles_total += results[i].NCycles * results[i].ExecutionCount;

	if (!loop_on) {
	    loop_start_PC = results[i].PC;
	    instructions_loop = 0;
	    packedcycles_loop = 0;
	    cycles_loop = 0;
	}
	if (results[i].loopstart) loop_on = 1;
	if (results[i].loopend) loop_on = 0;
	instructions_loop += (results[i].NInstructions *
			      results[i].ExecutionCount);
	packedcycles_loop += (results[i].NPackedCycles *
			      results[i].ExecutionCount);
	cycles_loop += results[i].NCycles * results[i].ExecutionCount;
	if (!loop_on) {
	    EnterLoop(loop_start_PC,results[i+1].PC-4,
		      instructions_loop,packedcycles_loop,cycles_loop);
	}
    }
}

void PrintTotal(long nBlocks) {
    PrintLoop();

    fprintf(file,"\n\ntotals:\n");
    fprintf(file,"                  instructions = %12ld\n",
	    instructions_total);
    fprintf(file,"   packed cycles no pipe delay = %12ld    cpi = %4.2f\n",
	    packedcycles_total,
	    packedcycles_total/(float)instructions_total);
    fprintf(file,"   wasted cycles (pipe stalls) = %12ld    cpi = %4.2f\n",
	    cycles_total-packedcycles_total,
	    (cycles_total-packedcycles_total)/(float)instructions_total);
    fprintf(file,"packed cycles with pipe stalls = %12ld    cpi = %4.2f\n",
	    cycles_total,
	    cycles_total/(float)instructions_total);
}
