/****************************************************************************
 *                                                                          *
 *  COPYRIGHT (c) 1994 BY                                                   *
 *  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.                  *
 *  ALL RIGHTS RESERVED.                                                    *
 *                                                                          *
 *  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   *
 *  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   *
 *  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY  OTHER   *
 *  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   *
 *  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   *
 *  TRANSFERRED.                                                            *
 *                                                                          *
 *  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   *
 *  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   *
 *  CORPORATION.                                                            *
 *                                                                          *
 *  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   *
 *  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
 *                                                                          *
 ****************************************************************************
 *              Copyright 1994 Digital Equipment Corporation
 *                         All Rights Reserved
 *
 * Permission to use, copy, and modify this software and its documentation is
 * hereby granted only under the following terms and conditions.  Both the
 * above copyright notice and this permission notice must appear in all copies
 * of the software, derivative works or modified versions, and any portions
 * thereof, and both notices must appear in supporting documentation.
 *
 * Users of this software agree to the terms and conditions set forth herein,
 * and hereby grant back to Digital a non-exclusive, unrestricted, royalty-
 * free right and license under any changes, enhancements or extensions 
 * made to the core functions of the software, including but not limited to 
 * those affording compatibility with other hardware or software 
 * environments, but excluding applications which incorporate this software.
 * Users further agree to use their best efforts to return to Digital any
 * such changes, enhancements or extensions that they make and inform Digital
 * of noteworthy uses of this software.  Correspondence should be provided
 * to Digital at:
 *
 *                       Director of Licensing
 *                       Western Research Laboratory
 *                       Digital Equipment Corporation
 *                       250 University Ave
 *                       Palo Alto, California  94301
 *
 * This software may be distributed (but not offered for sale or transferred
 * for compensation) to third parties, provided such third parties agree to
 * abide by the terms and conditions of this notice.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND DIGITAL EQUIPMENT CORP. DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL DIGITAL EQUIPMENT
 * CORPORATION BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */
#include <stdio.h>
#include <stdlib.h>
#include <instrument.h>
#include <assert.h>
#include "lpipe.h"

#define NClasses	(((long) ClassNull)+1)

int requiresResource[NClasses];

#define resourceI1 1
#define resourceI2 2
#define resourceM 4
#define resourceB 8

#define Requires(class, req) requiresResource[class] |= req

int DualIssue(Inst *instA,Inst *instB) {
/* return 1 if the two instructions can be dual issued (based on    */
/*  instruction classes, ignoring register conflicts)               */
/* return 0 if the two instructions can not be dual issued          */
    static int firstcall = 1;
    IClassType classA = GetInstClass(instA);
    IClassType classB = GetInstClass(instB);
    int dual;

    if (firstcall) {
	firstcall = 0;

	Requires(ClassFstore, resourceI1);
	Requires(ClassFbranch, resourceI1);
	Requires(ClassIarithmetic, resourceI1);
	Requires(ClassIlogical, resourceI1);
	Requires(ClassIshift, resourceI1);
	Requires(ClassIcompare, resourceI1);
	Requires(ClassIcondmove, resourceI1);
	Requires(ClassImultiplyl, resourceI1);
	Requires(ClassImultiplyq, resourceI1);
	
	Requires(ClassStore, resourceI2);
	Requires(ClassIbranch, resourceI2);
	Requires(ClassFpop, resourceI2);
	Requires(ClassFdivd, resourceI2);
	Requires(ClassFdivs, resourceI2);
	Requires(ClassSubroutine, resourceI2);
	Requires(ClassNull, resourceI2);	/* HW_x, CALL_PAL only */
	
	Requires(ClassLoad, resourceM);
	Requires(ClassFload, resourceM);
	Requires(ClassStore, resourceM);
	Requires(ClassFstore, resourceM);
	Requires(ClassNull, resourceM);
	Requires(ClassSubroutine, resourceM);
	
	Requires(ClassSubroutine, resourceB);
	Requires(ClassIbranch, resourceB);
	Requires(ClassFbranch, resourceB);
	Requires(ClassNull, resourceB);
    }
    dual = ((requiresResource[classA] & requiresResource[classB]) == 0);
#ifdef debug
    printf("0x%lx: %d %d %s\n", InstPC(instA), classA, classB,
	   (dual? "dual" : "single"));
#endif
    return dual;
}


#define max(a, b) (a > b ? a : b)

int maxdelay[NClasses], storedatadelay[NClasses], iaddusedelay[NClasses];

int Latency(Inst *instD, Inst *instU) {
/* Latency between Def and Use */
    static int firstcall = 1;
    int lat = 0;
    InstRegUsageVec bvecD;
    InstRegUsageVec bvecU;
    long dreg, ureg;

    IClassType classD = GetInstClass(instD);
    IClassType classU = GetInstClass(instU);

    if (firstcall) {
	firstcall = 0;
	maxdelay[ClassSubroutine] = 3;
	maxdelay[ClassLoad] = 3;
	maxdelay[ClassFload] = 3;
	maxdelay[ClassIcompare] = 2;
	maxdelay[ClassIarithmetic] = 2;
	maxdelay[ClassIlogical] = 2;
	maxdelay[ClassIshift] = 2;
	maxdelay[ClassIcondmove] = 2;
	maxdelay[ClassImultiplyl] = 21;
	maxdelay[ClassImultiplyq] = 23;
	maxdelay[ClassFpop] = 6;
	maxdelay[ClassFdivs] = 34;
	maxdelay[ClassFdivd] = 63;

	storedatadelay[ClassSubroutine] = 3;
	storedatadelay[ClassLoad] = 3;
	storedatadelay[ClassFload] = 3;
	storedatadelay[ClassImultiplyl] = 20;
	storedatadelay[ClassImultiplyq] = 22;
	storedatadelay[ClassFpop] = 4;
	storedatadelay[ClassFdivs] = 32;
	storedatadelay[ClassFdivd] = 61;

	iaddusedelay[ClassSubroutine] = 2;
	iaddusedelay[ClassIbranch] = 1;
	iaddusedelay[ClassStore] = 2;
	iaddusedelay[ClassLoad] = 2;
	iaddusedelay[ClassFload] = 2;
	iaddusedelay[ClassIcompare] = 1;
	iaddusedelay[ClassIarithmetic] = 1;
	iaddusedelay[ClassIlogical] = 1;
	iaddusedelay[ClassIshift] = 1;
	iaddusedelay[ClassIcondmove] = 1;
	iaddusedelay[ClassImultiplyl] = 1;
	iaddusedelay[ClassImultiplyq] = 1;
    }

    GetInstRegUsage(instD,&bvecD);
    GetInstRegUsage(instU,&bvecU);
    dreg = bvecD.dreg_bitvec[0] & ~0x8000000080000000;
    ureg = bvecU.ureg_bitvec[0];

    /* Only one integer multiply instruction at a time. */
    if (classD == ClassImultiplyl &&
	(classU == ClassImultiplyl || classU == ClassImultiplyq))
      lat = 19;
    if (classD == ClassImultiplyq &&
	(classU == ClassImultiplyl || classU == ClassImultiplyq))
      lat = 21;

    /* Only one divide instruction at a time. */
    if (classD == ClassFdivs && (classU == ClassFdivs || classU == ClassFdivd))
      lat = 30;
    if (classD == ClassFdivd && (classU == ClassFdivs || classU == ClassFdivd))
      lat = 59;

    if ((dreg & ureg) != 0) {
	if (classU == ClassStore || classU == ClassFstore) {
	    /* ST has different latencies for base register and store data */
	    int ra = GetInstInfo(instU,InstRA);
	    int rb = GetInstInfo(instU,InstRB);
	    if (dreg & (((long) 1) << rb)) lat = max(lat,maxdelay[classD]);
	    if (dreg & (((long) 1) << (ra + (classU == ClassFstore ? 32 : 0))))
	      lat = max(lat,storedatadelay[classD]);
	} else if (classD == ClassIarithmetic || classD == ClassIlogical) {
	    /* IADDLOG faster latencies to other integer ops */
	    lat = max(lat,iaddusedelay[classU]);
	} else if (classD == ClassIcompare && classU == ClassIbranch)  {
	    /* ICMP had one weird case */
	    lat = max(lat,1);
	} else {
	    lat = max(lat,maxdelay[classD]);
	}
    }
#ifdef debug
    printf("0x%lx (%d): %016lx  0x%lx: %016lx l=%d \n",
	   InstPC(instD), classD, dreg,
	   InstPC(instU), ureg, lat);
#endif
    return lat;
}



#define MAXCYCLESPERBLOCK 8000

Inst *instTable[MAXCYCLESPERBLOCK][2];
int activeTable[MAXCYCLESPERBLOCK][2];

void PipelineStallBlockData21064(Block *block, BasicBlockData * data) {
    static int firstcall = 1;
    int i, j, k;
    Inst *previnst;
    Inst *inst;
    int earliest;
    int earliest0;
    int pending;

    long pc;
    int instructions;
    int packedcycles;
    int cycles;

    InstRegUsageVec bvec;
    int overhang;
    IClassType iclass;

    if (firstcall) {
	firstcall = 0;
	for (i = 0; i < MAXCYCLESPERBLOCK; i++) {
	    instTable[i][0] = 0;
	    activeTable[i][0] = 0;
	    instTable[i][1] = 0;
	    activeTable[i][1] = 0;
	}
    }

    instructions = 0;
    packedcycles = 0;
    cycles = 0;
    pending = 0;

    data->PC = InstPC(GetFirstInst(block));
    data->LineNumber = InstLineNo(GetFirstInst(block));

    for (inst = GetFirstInst(block); inst != NULL; 
	 inst = GetNextInst(inst)) {
	int md, ad;
	instructions++;
	pc = InstPC(inst);
	earliest = cycles;
	packedcycles++;
	if (instructions > 1 && ((pc & 4) == 4)) {
	    if (DualIssue(previnst,inst)) {earliest--; packedcycles--;}
	}
	earliest0 = earliest;
	
	for (i = pending; i < cycles; i++) {
	    assert(i < MAXCYCLESPERBLOCK);
	    for (j = 0; j < 2; j++) {
		if (activeTable[i][j] == 0) continue;
		md = maxdelay[GetInstClass(instTable[i][j])];
		if (i+md < earliest) {
		    activeTable[i][j] = 0;
		    continue;
		}
		ad = Latency(instTable[i][j], inst);
		earliest = max(earliest, i+ad);
	    }
	    if (i == pending && activeTable[i][0] == 0 &&
		activeTable[i][1] == 0) pending++;
	}
#ifdef debug
	if (earliest != earliest0) {
	    printf("0x%lx (%02x): waste = %d \n",
		   InstPC(inst), GetInstInfo(inst,InstOpcode),
		   earliest-earliest0);
	}
#endif
	if (instTable[earliest][0] == 0) {
	    instTable[earliest][0] = inst;
	    activeTable[earliest][0] = 1;
	} else {
	    instTable[earliest][1] = inst;
	    activeTable[earliest][1] = 1;
	}
	cycles = earliest + 1;
	previnst = inst;
    }

#ifdef PRECISE
    for (i = 0; i < cycles; i++) {
        assert(i < MAXCYCLESPERBLOCK);
	for (j = 0; j < 2; j++) {
	    if (instTable[i][j] == 0) continue;
	    iclass = GetInstClass(instTable[i][j]);
	    if (iclass == ClassImultiplyl || iclass == ClassImultiplyq) {
		if (!data->FirstRead[UNIT_Integer_Multiply].Used) {
		    data->FirstRead[UNIT_Integer_Multiply].Delay = i;
		    data->FirstRead[UNIT_Integer_Multiply].Used = TRUE;
		}
	    } else if (iclass == ClassFdivs || iclass == ClassFdivd) {
		if (!data->FirstRead[UNIT_Float_Divide].Used) {
		    data->FirstRead[UNIT_Float_Divide].Delay = i;
		    data->FirstRead[UNIT_Float_Divide].Used = TRUE;
		}
	    }
	    GetInstRegUsage(instTable[i][j],&bvec);
	    for (k = 0; k < 64; k++) {
		if (bvec.ureg_bitvec[0] & (((long) 1) << k)) {
		    if (!data->FirstRead[k].Used) {
			data->FirstRead[k].Delay = i;
			data->FirstRead[k].Used = TRUE;
		    }
		}
	    }
	}
    }

    for (i = pending; i < cycles; i++) {
	for (j = 0; j < 2; j++) {
	    if (activeTable[i][j] == 0) continue;
	    iclass = GetInstClass(instTable[i][j]);
	    overhang = (i + maxdelay[iclass]) - cycles;
	    if (overhang <= 0) continue;
	    if (iclass == ClassImultiplyl) {
		data->OverhangingWrite[UNIT_Integer_Multiply].Delay =
		  (i + 19) - cycles;
		data->OverhangingWrite[UNIT_Integer_Multiply].Used = TRUE;
	    } else if (iclass == ClassImultiplyq) {
		data->OverhangingWrite[UNIT_Integer_Multiply].Delay =
		  (i + 21) - cycles;
		data->OverhangingWrite[UNIT_Integer_Multiply].Used = TRUE;
	    } else if (iclass == ClassFdivs) {
		data->OverhangingWrite[UNIT_Float_Divide].Delay =
		  (i + 30) - cycles;
		data->OverhangingWrite[UNIT_Float_Divide].Used = TRUE;
	    } else if (iclass == ClassFdivd) {
		data->OverhangingWrite[UNIT_Float_Divide].Delay =
		  (i + 59) - cycles;
		data->OverhangingWrite[UNIT_Float_Divide].Used = TRUE;
	    }
	    GetInstRegUsage(instTable[i][j],&bvec);
	    for (k = 0; k < 64; k++) {
		if (bvec.dreg_bitvec[0] & (((long) 1) << k)) {
		    data->OverhangingWrite[k].Delay = overhang;
		    data->OverhangingWrite[k].Used = TRUE;
		}
	    }
	}
    }
#endif PRECISE

    data->NInstructions = instructions;
    data->NPackedCycles = packedcycles;
    data->NCycles = cycles;

    for (i = 0; i < cycles; i++) {
	instTable[i][0] = 0;
	activeTable[i][0] = 0;
	instTable[i][1] = 0;
	activeTable[i][1] = 0;
    }
}

#define BLOCKS_PER_PROC 5000
#define NESTEDLOOP_PER_PROC 100

long firstPC[BLOCKS_PER_PROC];
int branchout[BLOCKS_PER_PROC];
int branchin[BLOCKS_PER_PROC];
int maxbranch[BLOCKS_PER_PROC];
int branchesthrough[BLOCKS_PER_PROC][NESTEDLOOP_PER_PROC];

Instrument() {
   Proc *proc;
   Block *block;
   FILE *file;
   BasicBlockData *results;

   long number,numberFirst;
   int instructions;
   int packedcycles;
   int cycles;
   char openProto[200];

   BasicBlockData blockdata;

   file = fopen("pipe.static","w");

   AddCallProto("BlockCount(long)");
   AddCallProto("PrintHeader(long)");
   AddCallProto("PrintProc(long,long,char *,char *)");
   AddCallProto("PrintTotal()");
   AddCallProto("CloseFile()");

   results = (BasicBlockData *) malloc(sizeof(BasicBlockData) *
				       GetProgramInfo(ProgramNumberBlocks));
   if (results == NULL)
     {
       fprintf(stderr,"pipe: malloc failed\n");
       exit(1);
     }
   bzero(results,sizeof(BasicBlockData) * GetProgramInfo(ProgramNumberBlocks));

   AddCallProgram(ProgramAfter,"PrintHeader",
		  GetProgramInfo(ProgramNumberBlocks));

   number = 0;   
   for (proc = GetFirstProc(); proc != NULL; proc = GetNextProc(proc)) {
       int nblocks, bk;
       long target;
       int found, maxb, j, k ,l;

       fprintf(file,
	   "--------------------------------------------------------------\n");
       fprintf(file,"%s : %s\n",ProcName(proc),ProcFileName(proc));
       fprintf(file,
           "--------------------------------------------------------------\n");
       fprintf(file,"  %5s %11s %4s %4s %4s   %4s\n",
	      "Line", "PC", "Inst", "Pack", "Cycl", "CPI");

       nblocks = 0;
       for (block = GetFirstBlock(proc); block != NULL; 
                                         block = GetNextBlock(block)) {
	   firstPC[nblocks] = BlockPC(block);
	   branchin[nblocks] = -1;
	   branchout[nblocks] = -1;
	   maxbranch[nblocks] = -1;
	   nblocks++;
       }
       bk = -1;
       for (block = GetFirstBlock(proc); block != NULL; 
                                         block = GetNextBlock(block)) {
	   bk++;
	   if (!IsInstType(GetLastInst(block),InstTypeCondBr)) continue;
	   target = InstPC(GetInstBranchTarget(GetLastInst(block)));
	   if (target < firstPC[0]) continue;
	   if (target > firstPC[bk]) continue;

	   found = -1;
	   maxb = -1;
	   for (j = 0; j <= bk; j++) {
	       if (target == firstPC[j]) found = j;
	       if (found != -1) maxb = max(maxb,maxbranch[j]);
	   }
	   maxb++;
	   branchin[found] = maxb;
	   branchout[bk] = maxb;
	   for (k = found; k <= bk; k++) {
	       for (l = maxbranch[k]+1; l < maxb; l++) 
		 branchesthrough[k][l] = 0;
	       maxbranch[k] = max(maxbranch[k], maxb);
	       branchesthrough[k][maxb] = 1;
	   }
	   branchesthrough[found][maxb] = 2;
	   branchesthrough[bk][maxb] = 2;
       }

       numberFirst = number;
       bk = 0;
       for (block = GetFirstBlock(proc); block != NULL; 
	    block = GetNextBlock(block)) {
	   PipelineStallBlockData21064(block, &results[number]);
	   fprintf(file,"  %5d 0x%9lx %4d %4d %4d  %5.2f  ",
		   results[number].LineNumber,
		   results[number].PC, results[number].NInstructions, 
		   results[number].NPackedCycles,
		   results[number].NCycles,
		   results[number].NCycles / 
		   ((float) results[number].NInstructions));
	   if (branchin[bk] != -1 && branchout[bk] != -1) fprintf(file,"<>");
	   else if (branchin[bk] != -1) fprintf(file,"<-");
	   else if (branchout[bk] != -1) fprintf(file,"->");
	   else if (maxbranch[bk] != -1) fprintf(file,"  ");
	   for (j = 0; j <= maxbranch[bk]; j++) {
	       if (branchesthrough[bk][j] == 2)		  fprintf(file,"-+");
	       else if (j<branchin[bk] || j<branchout[bk])fprintf(file,"--");
	       else if (branchesthrough[bk][j] == 1)	  fprintf(file," |");
	       else					  fprintf(file,"  ");
	   }
	   fprintf(file,"\n");

	   AddCallBlock(block,BlockBefore,"BlockCount",number);
	   bk++;
	   number++;
       }
       AddCallProgram(ProgramAfter,"PrintProc",numberFirst,number,
		      ProcName(proc),ProcFileName(proc));
   }   
   sprintf(openProto,"OpenFile(char [%d])",number*sizeof(BasicBlockData));
   AddCallProto(openProto);
   AddCallProgram(ProgramBefore,"OpenFile", results);
   AddCallProgram(ProgramAfter,"PrintTotal");
   AddCallProgram(ProgramAfter,"CloseFile");
   fclose(file);
}
