/****************************************************************************
 *                                                                          *
 *  COPYRIGHT (c) 1994 BY                                                   *
 *  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.                  *
 *  ALL RIGHTS RESERVED.                                                    *
 *                                                                          *
 *  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   *
 *  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   *
 *  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY  OTHER   *
 *  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   *
 *  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   *
 *  TRANSFERRED.                                                            *
 *                                                                          *
 *  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   *
 *  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   *
 *  CORPORATION.                                                            *
 *                                                                          *
 *  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   *
 *  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
 *                                                                          *
 ****************************************************************************
 *              Copyright 1994 Digital Equipment Corporation
 *                         All Rights Reserved
 *
 * Permission to use, copy, and modify this software and its documentation is
 * hereby granted only under the following terms and conditions.  Both the
 * above copyright notice and this permission notice must appear in all copies
 * of the software, derivative works or modified versions, and any portions
 * thereof, and both notices must appear in supporting documentation.
 *
 * Users of this software agree to the terms and conditions set forth herein,
 * and hereby grant back to Digital a non-exclusive, unrestricted, royalty-
 * free right and license under any changes, enhancements or extensions 
 * made to the core functions of the software, including but not limited to 
 * those affording compatibility with other hardware or software 
 * environments, but excluding applications which incorporate this software.
 * Users further agree to use their best efforts to return to Digital any
 * such changes, enhancements or extensions that they make and inform Digital
 * of noteworthy uses of this software.  Correspondence should be provided
 * to Digital at:
 *
 *                       Director of Licensing
 *                       Western Research Laboratory
 *                       Digital Equipment Corporation
 *                       250 University Ave
 *                       Palo Alto, California  94301
 *
 * This software may be distributed (but not offered for sale or transferred
 * for compensation) to third parties, provided such third parties agree to
 * abide by the terms and conditions of this notice.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND DIGITAL EQUIPMENT CORP. DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL DIGITAL EQUIPMENT
 * CORPORATION BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */
#ifndef PLATFORM
#define PLATFORM

#define WB_MAX            5
#define TB_MAX           32
#define LINE_MIN         32
#define PCACHE_MAX    16384
#define BCACHE_MAX  4194304
#define PAGE_SIZE      8192
#define PAGE_SHIFT       13

struct platformType {
  char  *model;        /* Model number                                   */
  int   itbSize;       /* User translation buffer entries                */
  int   dtbSize;       /* number of entries in the translation buffer    */
  float tbService;     /* Cycles to servie translation buffer miss       */
  int   wbSize;        /* number of entries in the on-chip write buff    */
  int   icacheSize;    /* Size(bytes) of the on chip instruction cache   */
  int   icacheShift;   /* log 2 of the instruction cache line size       */
  int   dcacheSize;    /* Size(bytes) of the on chip data cache          */
  int   dcacheShift;   /* log 2 of the data cache line size              */
  int   bcacheSize;    /* Size(bytes) of the board cache                 */
  int   bcacheShift;   /* log 2 of the board cache line size             */
  float cycleTime;     /* Processor cycle time in nanoseconds            */
  float iMissService;  /* Average service time of instruction cache miss */
  float dMissService;  /* Average service time of data cache miss        */
  float dWriteService; /* Average service time of data cache board write */
  float bMissService;  /* Average service time of board cache miss       */
  float bVictimService;/* Average service time of victim only            */
} platforms[] = {
/*
 *  first item acts as the default
 */            /* chip specific paramteters                   */  /* platform specific parameters */
"DEC3000_500", 8, 32, 50.0, 4, 8192, 5, 8192, 5, 524288, 5, 6.66,  66.6,  66.6,  99.9, 260.0, 190.0,
"DEC3000_300", 8, 32, 50.0, 4, 8192, 5, 8192, 5, 262144, 5, 6.66, 106.6, 106.6, 133.3, 450.0, 400.0,
"DEC3000_400", 8, 32, 50.0, 4, 8192, 5, 8192, 5, 524288, 5, 7.50,  75.0,  75.0, 112.5, 292.5, 207.5,
"DEC3000_600", 8, 32, 50.0, 4, 8192, 5, 8192, 5,2097152, 5, 5.71,  57.1,  57.1,  85.7, 260.0, 190.0,
"DEC4000",     8, 32, 50.0, 4, 8192, 5, 8192, 5,1048576, 5, 6.25,  50.0,  50.0,  75.0, 182.0,   0.0,
"DEC7000",     8, 32, 50.0, 4, 8192, 5, 8192, 5,4194304, 6, 5.50,  82.5,  82.5,  82.5, 400.0,   0.0,
"DEC10000",    8, 32, 50.0, 4, 8192, 5, 8192, 5,4194304, 6, 5.00,  75.0,  75.0,  75.0, 400.0,   0.0,
           "", 0,  0,  0.0, 0,    0, 0,    0, 0,      0, 0,  0.0,   0.0,   0.0,   0.0,   0.0,   0.0
	   };

typedef struct platformType PlatformType;

#endif PLATFORM

