/****************************************************************************
 *                                                                          *
 *  COPYRIGHT (c) 1994 BY                                                   *
 *  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.                  *
 *  ALL RIGHTS RESERVED.                                                    *
 *                                                                          *
 *  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   *
 *  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   *
 *  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY  OTHER   *
 *  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   *
 *  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   *
 *  TRANSFERRED.                                                            *
 *                                                                          *
 *  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   *
 *  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   *
 *  CORPORATION.                                                            *
 *                                                                          *
 *  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   *
 *  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
 *                                                                          *
 ****************************************************************************
 *              Copyright 1994 Digital Equipment Corporation
 *                         All Rights Reserved
 *
 * Permission to use, copy, and modify this software and its documentation is
 * hereby granted only under the following terms and conditions.  Both the
 * above copyright notice and this permission notice must appear in all copies
 * of the software, derivative works or modified versions, and any portions
 * thereof, and both notices must appear in supporting documentation.
 *
 * Users of this software agree to the terms and conditions set forth herein,
 * and hereby grant back to Digital a non-exclusive, unrestricted, royalty-
 * free right and license under any changes, enhancements or extensions 
 * made to the core functions of the software, including but not limited to 
 * those affording compatibility with other hardware or software 
 * environments, but excluding applications which incorporate this software.
 * Users further agree to use their best efforts to return to Digital any
 * such changes, enhancements or extensions that they make and inform Digital
 * of noteworthy uses of this software.  Correspondence should be provided
 * to Digital at:
 *
 *                       Director of Licensing
 *                       Western Research Laboratory
 *                       Digital Equipment Corporation
 *                       250 University Ave
 *                       Palo Alto, California  94301
 *
 * This software may be distributed (but not offered for sale or transferred
 * for compensation) to third parties, provided such third parties agree to
 * abide by the terms and conditions of this notice.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND DIGITAL EQUIPMENT CORP. DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL DIGITAL EQUIPMENT
 * CORPORATION BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "platform.h"

#define SYSCALLS 259

struct SyscallType {
  long calls;
  long total;
  long context;
} syscalls[SYSCALLS];

long ccPrev;
int callPrev;
long context;

struct {
  int call;
  char *syscall;
} mapSyscall[SYSCALLS] =
{
  0,  "syscall",
  1,  "exit",
  2,  "fork",
  3,  "read",
  4,  "write",
  5,  "old_open",
  6,  "close",
  7,  "wait4",
  8,  "old_creat",
  9,  "link",
  10, "unlink",
  11, "execv",
  12, "chdir",
  13, "fchdir",
  14, "mknod",
  15, "chmod",
  16, "chown",
  17, "obreak",
  18, "getfsstat",
  19, "lseek",
  20, "getpid",
  21, "mount",
  22, "unmount",
  23, "setuid",
  24, "getuid",
  25, "exec_ld",
  26, "ptrace",
  27, "recvmsg",
  28, "sendmsg",
  29, "recvfrom",
  30, "accept",
  31, "getpeername",
  32, "getsockname",
  33, "access",
  34, "chflags",
  35, "fchflags",
  36, "sync",
  37, "kill",
  38, "old_stat",
  39, "setpgid",
  40, "old_lstat",
  41, "dup",
  42, "pipe",
  43, "set_pattr",
  44, "profil",
  45, "open",
  46, "sigaction",
  47, "getgid",
  48, "sigprocmask",
  49, "getlogin",
  50, "setlogin",
  51, "acct",
  52, "sigpending",
  53, "sigaltstack",
  54, "ioctl",
  55, "reboot",
  56, "revoke",
  57, "symlink",
  58, "readlink",
  59, "execve",
  60, "umask",
  61, "chroot",
  62, "old_fstat",
  63, "getpgrp",
  64, "getpagesize",
  65, "mremap",
  66, "vfork",
  67, "stat",
  68, "lstat",
  69, "sbrk",
  70, "sstk",
  71, "mmap",
  72, "ovadvise",
  73, "munmap",
  74, "mprotect",
  75, "madvise",
  76, "old_vhangup",
  77, "kmodcall",
  78, "mincore",
  79, "getgroups",
  80, "setgroups",
  81, "old_getpgrp",
  82, "setpgrp",
  83, "setitimer",
  84, "old_wait",
  85, "table",
  86, "getitimer",
  87, "gethostname",
  88, "sethostname",
  89, "getdtablesize",
  90, "dup2",
  91, "fstat",
  92, "fcntl",
  93, "select",
  94, "poll",
  95, "fsync",
  96, "setpriority",
  97, "socket",
  98, "connect",
  99, "accept",
  100,"getpriority",
  101,"send",
  102,"recv",
  103,"sigreturn",
  104,"bind",
  105,"setsockopt",
  106,"listen",
  107,"plock",
  108,"old_sigvec",
  109,"old_sigblock",
  110,"old_sigsetmask",
  111,"sigsuspend",
  112,"sigstack",
  113,"recvmsg",
  114,"sendmsg",
  115,"vtrace",
  116,"gettimeofday",
  117,"getrusage",
  118,"getsockopt",
  119,"resuba",
  120,"readv",
  121,"writev",
  122,"settimeofday",
  123,"fchown",
  124,"fchmod",
  125,"recvfrom",
  126,"setreuid",
  127,"setregid",
  128,"rename",
  129,"truncate",
  130,"ftruncate",
  131,"flock",
  132,"setgid",
  133,"sendto",
  134,"shutdown",
  135,"socketpair",
  136,"mkdir",
  137,"rmdir",
  138,"utimes",
  139,"old_sigreturn",
  140,"adjtime",
  141,"getpeername",
  142,"gethostid",
  143,"sethostid",
  144,"getrlimit",
  145,"setrlimit",
  146,"old_killpg",
  147,"setsid",
  148,"quotactl",
  149,"oldquota",
  150,"getsockname",
  151,"exect",
  152,"unknown",
  153,"getfpemulator",
  154,"old_iopen",
  155,"old_iread",
  156,"sigaction",
  157,"unknown",
  158,"nfs_svc",
  159,"getdirentries",
  160,"statfs",
  161,"fstatfs",
  162,"unknown",
  163,"aysnc_daemon",
  164,"getfh",
  165,"getdomainname",
  166,"setdomainname",
  167,"unknown",
  168,"unknown",
  169,"exportfs",
  170,"unknown",
  171,"unknown",
  172,"msgctl",
  173,"msgget",
  174,"msgrcv",
  175,"msgsnd",
  176,"semctl",
  177,"semget",
  178,"semop",
  179,"uname",
  180,"shmsys",
  181,"plock",
  182,"lockf",
  183,"ustat",
  184,"getmnt",
  185,"unknown",
  186,"unknown",
  187,"sigpending",
  188,"setsid",
  189,"waitpid",
  190,"unknown",
  191,"unknown",
  192,"unknown",
  193,"unknown",
  194,"unknown",
  195,"unknown",
  196,"unknown",
  197,"unknown",
  198,"unknown",
  199,"swapon",
  200,"msgctl",
  201,"msgget",
  202,"msgrcv",
  203,"msgsnd",
  204,"semctl",
  205,"semget",
  206,"semop",
  207,"uname",
  208,"lchown",
  209,"shmat",
  210,"shmctl",
  211,"shmdt",
  212,"shmget",
  213,"mvalid",
  214,"getaddressconf",
  215,"msleep",
  216,"mwakeup",
  217,"msync",
  218,"signal",
  219,"utc_gettime",
  220,"utc_adjtime",
  221,"unknown",
  222,"unknown",
  223,"kloadcall",
  224,"pioctl",
  225,"setpag",
  226,"iopen",
  227,"icreate",
  228,"iread",
  229,"iwrite",
  230,"iinc",
  231,"idec",
  232,"afs_call",
  233,"getpgid",
  234,"getsid",
  235,"unknown",
  236,"unknown",
  237,"unknown",
  238,"unknown",
  239,"unknown",
  240,"msfs_syscall",
  241,"unknown",
  242,"unknown",
  243,"unknown",
  244,"unknown",
  245,"unknown",
  246,"unknown",
  247,"unknown",
  248,"unknown",
  249,"unknown",
  250,"uswitch",
  251,"usleep_thread",
  252,"audcntl",
  253,"audgen",
  254,"startcpu",
  255,"stopcpu",
  256,"getsysinfo",
  257,"setsysinfo",
  258, "afs_syscall"
  };
  
void BeforePal(long call, long cc)
{
  if (cc >> 32 != ccPrev >> 32) context++;
  ccPrev = cc;
  callPrev = call;
}

void AfterPal(long cc)
{
  unsigned int ccInt,ccIntPrev;

  ccInt = cc;
  ccIntPrev = ccPrev;
  if (callPrev >= 0 && callPrev <= 258)
    {
      syscalls[callPrev].total += ccInt - ccIntPrev;
      syscalls[callPrev].calls++;
      if (cc >> 32 != ccPrev >> 32) syscalls[callPrev].context++;
    }
}

FILE *OpenUnique(char *fileName, char *type)
{
  FILE *file;
  char Name[200];

  if (getenv("ATOMUNIQUE") != NULL)
    sprintf(Name,"%s.%d",fileName,getpid());
  else
    strcpy(Name,fileName);

  file = fopen(Name,type);
  if (file == NULL)
    {
      fprintf(stderr,"Atom: can't open %s for %s\n",Name, type);
      exit(1);
    }
  return(file);
}

FILE *file;

void GetPlatform(PlatformType *platform)
{
  FILE *filePlatform;
  char model[100];
  int i;

  filePlatform = popen("/usr/sbin/sizer -c","r");
  assert(filePlatform != NULL);
  fscanf(filePlatform,"cpu \"%[^\"]s",model);
  for (i = 0; strcmp(model,platforms[i].model)!=0 &&
              platforms[i].model[0] != '\0'; i++);
  if (platforms[i].model[0] == '\0')
    {
      fprintf(file,"Model %s not found\n",model);
      i = 0;
    }
  fprintf(file,"Using model %s platform parameters\n\n",platforms[i].model);
  *platform = platforms[i];
  fclose(filePlatform);
}

PlatformType platform;

void OpenFile()
{
  int i;
  long total = 0;

  file = OpenUnique("syscall.out","w");
  GetPlatform(&platform);
}

void PrintSysCalls()
{
  int i;
  long total = 0;

  fprintf(file,"%25s %10s   %15s %10s\n",
	  "System Call","Calls","Time(ms)","context");

  for ( i=0; i < SYSCALLS; i++)
    {
      if (syscalls[i].total != 0)
	{
	  total += syscalls[i].total;
	  if (strcmp(mapSyscall[i].syscall,"unknown")==0)
	    {
	      fprintf(file,"%22s%3d %10ld %15.1f %10ld\n", 
		      mapSyscall[i].syscall,
		      i,syscalls[i].calls,
		      (platform.cycleTime * syscalls[i].total) / 1000000.0,
		      syscalls[i].context);
	    }
	  else
	    {
	      fprintf(file,"%25s %10ld %15.1f %10ld\n", 
                      mapSyscall[i].syscall, syscalls[i].calls,
		      (platform.cycleTime * syscalls[i].total) / 1000000.0,
		      syscalls[i].context);
	    }
	}
    }
  fprintf(file,"\n%25s %10s %15.1f %10ld\n",
	  "Total","", (platform.cycleTime * total) / 1000000.0, context);
  fclose(file);
}
