/****************************************************************************
 *                                                                          *
 *  COPYRIGHT (c) 1994 BY                                                   *
 *  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.                  *
 *  ALL RIGHTS RESERVED.                                                    *
 *                                                                          *
 *  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   *
 *  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   *
 *  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY  OTHER   *
 *  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   *
 *  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   *
 *  TRANSFERRED.                                                            *
 *                                                                          *
 *  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   *
 *  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   *
 *  CORPORATION.                                                            *
 *                                                                          *
 *  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   *
 *  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
 *                                                                          *
 ****************************************************************************
 *              Copyright 1994 Digital Equipment Corporation
 *                         All Rights Reserved
 *
 * Permission to use, copy, and modify this software and its documentation is
 * hereby granted only under the following terms and conditions.  Both the
 * above copyright notice and this permission notice must appear in all copies
 * of the software, derivative works or modified versions, and any portions
 * thereof, and both notices must appear in supporting documentation.
 *
 * Users of this software agree to the terms and conditions set forth herein,
 * and hereby grant back to Digital a non-exclusive, unrestricted, royalty-
 * free right and license under any changes, enhancements or extensions 
 * made to the core functions of the software, including but not limited to 
 * those affording compatibility with other hardware or software 
 * environments, but excluding applications which incorporate this software.
 * Users further agree to use their best efforts to return to Digital any
 * such changes, enhancements or extensions that they make and inform Digital
 * of noteworthy uses of this software.  Correspondence should be provided
 * to Digital at:
 *
 *                       Director of Licensing
 *                       Western Research Laboratory
 *                       Digital Equipment Corporation
 *                       250 University Ave
 *                       Palo Alto, California  94301
 *
 * This software may be distributed (but not offered for sale or transferred
 * for compensation) to third parties, provided such third parties agree to
 * abide by the terms and conditions of this notice.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND DIGITAL EQUIPMENT CORP. DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL DIGITAL EQUIPMENT
 * CORPORATION BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */
#include <stdio.h>
#include <assert.h>
#include "platform.h"
#include "tb.h"

#define NEXTITB(index)  (((index)+1) & (platform.itbSize-1))
#define NEXTDTB(index)  (((index)+1) & (platform.dtbSize-1))
#define STRLEN  200

struct TransBuffer {
  long page;
  long valid;
} dtb[TB_MAX],itb[TB_MAX];

#define MAPSIZE 16384

struct {
  int next;
  int page[MAPSIZE];
} itbMap,dtbMap;

FILE *file;

void GetPlatform(PlatformType *platform)
{
  FILE *filePlatform;
  char model[100];
  int i;

  filePlatform = popen("/usr/sbin/sizer -c","r");
  assert(filePlatform != NULL);
  fscanf(filePlatform,"cpu \"%[^\"]s",model);
  for (i = 0; strcmp(model,platforms[i].model)!=0 &&
              platforms[i].model[0] != '\0'; i++);
  if (platforms[i].model[0] == '\0')
    {
      fprintf(file,"Model %s not found\n",model);
      i = 0;
    }
  fprintf(file,"Using model %s platform parameters\n\n",platforms[i].model);
  *platform = platforms[i];
  fclose(filePlatform);
}

extern int errno;
FILE *OpenUnique(char *fileName, char *type)
{
  FILE *file;
  char name[STRLEN];

  if (getenv("ATOMUNIQUE") != NULL)
    sprintf(name,"%s.%d",fileName,getpid());
  else
    strcpy(name,fileName);

  file = fopen(name,type);
  if (file == NULL)
    {
      fprintf(stderr,"Atom: can't open %s for %s: %d\n",name, type, errno);
      exit(1);
    }
  return(file);
}

PlatformType platform;
struct TbMissType *tbMisses;

char buf[BUFSIZ];
void OpenFile(char *data)
{
  tbMisses = (struct TbMissType *) data;
  file = OpenUnique("tb.out","w");
  assert(file != NULL);
  setbuffer(file,buf,BUFSIZ);
  GetPlatform(&platform);
  fprintf(file,"%30s %15s %15s %15s %15s %15s\n", "Procedure","ITB Misses",
	  "ITB time(ms)","DTB Misses","DTB Time(ms)","Total Time(ms)");
}

void Instruction(int number,long address)
{
  long page;
  int i;

  page = address >> PAGE_SHIFT;
  i = itbMap.page[page & (MAPSIZE-1)];
  if (!itb[i].valid || itb[i].page != page)
    {
      for (i = 0; i < platform.itbSize; i++)
	{
	  if (itb[i].valid && itb[i].page == page)
	    {
	      itbMap.page[page & (MAPSIZE-1)] = i;
	      break;
	    }
	}
    }
  if (i >= platform.itbSize)
    {
      tbMisses[number].inst++;
      itb[itbMap.next].page = page;
      itb[itbMap.next].valid = 1;
      itbMap.page[page & (MAPSIZE-1)] = itbMap.next;
      itbMap.next = NEXTITB(itbMap.next);
    }
  else
    {
      if (itbMap.next == i)
	{
	  itbMap.next = NEXTITB(i);
	}
    }
}

void Reference(int number,long address)
{
  long page;
  int i;

  page = address >> PAGE_SHIFT;
  i = dtbMap.page[page & (MAPSIZE-1)];
  if (!dtb[i].valid || dtb[i].page != page)
    {
      for (i = 0; i < platform.dtbSize; i++)
	{
	  if (dtb[i].valid && dtb[i].page == page)
	    {
	      dtbMap.page[page & (MAPSIZE-1)] = i;
	      break;
	    }
	}
    }
  if (i >= platform.dtbSize)
    {
      tbMisses[number].data++;
      dtb[dtbMap.next].page = page;
      dtb[dtbMap.next].valid = 1;
      dtbMap.page[page & (MAPSIZE-1)] = dtbMap.next;
      dtbMap.next = NEXTDTB(dtbMap.next);
    }
  else
    {
      if (dtbMap.next == i)
	{
	  dtbMap.next = NEXTDTB(i);
	}
    }
}

long totalItbMiss, itbOther;
long totalDtbMiss, dtbOther;

void PrintResults(int number, char *name)
{

  float itbService,dtbService;

  itbService = tbMisses[number].inst * platform.tbService * platform.cycleTime
               / 1000000.0;
  dtbService = tbMisses[number].data * platform.tbService * platform.cycleTime
               / 1000000.0;
  if (itbService >= 0.1 || dtbService >= 0.1)
    {
      fprintf(file,"%30s %15ld %15.1f %15ld %15.1f %15.1f\n", name,
	    tbMisses[number].inst, itbService,
	    tbMisses[number].data, dtbService,
	    itbService+dtbService);
    }
  else
    {
      itbOther += tbMisses[number].inst;
      dtbOther += tbMisses[number].data;
    }
    
  totalItbMiss += tbMisses[number].inst;
  totalDtbMiss += tbMisses[number].data;
}

void CloseFile()
{
  float itbService, dtbService;

  itbService = (itbOther * platform.tbService * platform.cycleTime )
                       / 1000000.0;
  dtbService = (dtbOther * platform.tbService * platform.cycleTime )
                       / 1000000.0;
  fprintf(file,"\n%30s %15ld %15.1f %15ld %15.1f %15.1f\n", "Other",
	  itbOther,itbService,
	  dtbOther,dtbService,
	  itbService+dtbService);

  itbService = (totalItbMiss * platform.tbService * platform.cycleTime )
                        / 1000000.0;
  dtbService = (totalDtbMiss * platform.tbService * platform.cycleTime )
                        / 1000000.0;
  fprintf(file,"\n%30s %15ld %15.1f %15ld %15.1f %15.1f\n", "Total",
	  totalItbMiss, itbService,
	  totalDtbMiss, dtbService,
	  itbService+dtbService);
  fclose(file);
}
