/****************************************************************************
 *                                                                          *
 *  COPYRIGHT (c) 1994 BY                                                   *
 *  DIGITAL EQUIPMENT CORPORATION, MAYNARD, MASSACHUSETTS.                  *
 *  ALL RIGHTS RESERVED.                                                    *
 *                                                                          *
 *  THIS SOFTWARE IS FURNISHED UNDER A LICENSE AND MAY BE USED AND COPIED   *
 *  ONLY IN  ACCORDANCE WITH  THE  TERMS  OF  SUCH  LICENSE  AND WITH THE   *
 *  INCLUSION OF THE ABOVE COPYRIGHT NOTICE.  THIS SOFTWARE OR ANY  OTHER   *
 *  COPIES THEREOF MAY NOT BE PROVIDED OR OTHERWISE MADE AVAILABLE TO ANY   *
 *  OTHER PERSON.  NO TITLE TO AND OWNERSHIP OF  THE  SOFTWARE IS  HEREBY   *
 *  TRANSFERRED.                                                            *
 *                                                                          *
 *  THE INFORMATION IN THIS SOFTWARE IS  SUBJECT TO CHANGE WITHOUT NOTICE   *
 *  AND  SHOULD  NOT  BE  CONSTRUED AS  A COMMITMENT BY DIGITAL EQUIPMENT   *
 *  CORPORATION.                                                            *
 *                                                                          *
 *  DIGITAL ASSUMES NO RESPONSIBILITY FOR THE USE  OR  RELIABILITY OF ITS   *
 *  SOFTWARE ON EQUIPMENT WHICH IS NOT SUPPLIED BY DIGITAL.                 *
 *                                                                          *
 ****************************************************************************
 *              Copyright 1994 Digital Equipment Corporation
 *                         All Rights Reserved
 *
 * Permission to use, copy, and modify this software and its documentation is
 * hereby granted only under the following terms and conditions.  Both the
 * above copyright notice and this permission notice must appear in all copies
 * of the software, derivative works or modified versions, and any portions
 * thereof, and both notices must appear in supporting documentation.
 *
 * Users of this software agree to the terms and conditions set forth herein,
 * and hereby grant back to Digital a non-exclusive, unrestricted, royalty-
 * free right and license under any changes, enhancements or extensions 
 * made to the core functions of the software, including but not limited to 
 * those affording compatibility with other hardware or software 
 * environments, but excluding applications which incorporate this software.
 * Users further agree to use their best efforts to return to Digital any
 * such changes, enhancements or extensions that they make and inform Digital
 * of noteworthy uses of this software.  Correspondence should be provided
 * to Digital at:
 *
 *                       Director of Licensing
 *                       Western Research Laboratory
 *                       Digital Equipment Corporation
 *                       250 University Ave
 *                       Palo Alto, California  94301
 *
 * This software may be distributed (but not offered for sale or transferred
 * for compensation) to third parties, provided such third parties agree to
 * abide by the terms and conditions of this notice.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND DIGITAL EQUIPMENT CORP. DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL DIGITAL EQUIPMENT
 * CORPORATION BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */
#include <stdio.h>
#include <instrument.h>

Instrument()
{
   Proc *proc;
   Block *block;
   Inst *inst;
   int procNumber;
   int instructions,instructionsByte;
   InstRegUsageVec usageVec;
   unsigned long shadow[2];


   AddCallProto("OpenFile(int)");
   AddCallProto("ProcedureCount(int,int,int)");
   AddCallProto("ProcedurePrint(int,char*)");
   AddCallProto("CloseFile()");

   procNumber = 0;
   for (proc = GetFirstProc(); proc != NULL; proc = GetNextProc(proc))
     {
       for (block = GetFirstBlock(proc); block != NULL; 
                                         block = GetNextBlock(block))
         {
	   shadow[0] = shadow[1] = 0;
	   instructions = 0;
	   instructionsByte = 0;
	   for (inst = GetFirstInst(block); inst != NULL; 
		                             inst = GetNextInst(inst))
	     {
	       GetInstRegUsage(inst,&usageVec);

	       /* load quadword unaligned */
	       if (GetInstInfo(inst,InstOpcode) == 0xb ||
		   shadow[0] & usageVec.ureg_bitvec[0] ||
		   shadow[1] & usageVec.ureg_bitvec[1])
		 {
		   shadow[0] |= usageVec.dreg_bitvec[0];
		   shadow[1] |= usageVec.dreg_bitvec[1];
		   instructionsByte++;
		 }
	       else
		 {
		   shadow[0] &= ~usageVec.dreg_bitvec[0];
		   shadow[1] &= ~usageVec.dreg_bitvec[1];
		 }
	       instructions++;
	     }
	   AddCallBlock(block,BlockBefore,"ProcedureCount",
			 procNumber,instructions,instructionsByte);
         }
       AddCallProgram(ProgramAfter,"ProcedurePrint",procNumber,ProcName(proc));
       procNumber++;
     }
   AddCallProgram(ProgramBefore,"OpenFile",procNumber);
   AddCallProgram(ProgramAfter,"CloseFile");
}

